#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>

#include "liste_ch_int_float.h"

struct s_graphe_listes_succ_p {
	int n; //nombre de sommets
	liste_c* tab; // tableau de n listes chaînées de successeurs pondérés
};

typedef struct s_graphe_listes_succ_p gra_lsp;


gra_lsp* cree_graphe_vide (int nb_sommets){
	// renvoie l'adresse d'une nouvelle structure de graphe par listes 
	// contenant un graphe à nb_sommets sommets mais aucun arc 
	gra_lsp* res = (gra_lsp*) malloc ( sizeof(gra_lsp));
	res->n = nb_sommets;
	res->tab = (liste_c*) malloc(sizeof(liste_c*) *nb_sommets );
	for (int i = 0; i < res->n; i++){
		(res->tab)[i] = NULL; //liste chaînée vide
	}
	return res;
}




void supprime_graphe (gra_lsp* pg){
	// libère l'espace mémoire occupé par me graphe *pg
	for (int i = 0; i < pg->n; i++){
		if ((pg->tab)[i] != NULL)
			free_liste_c((pg->tab)[i]);
	}
	free(pg->tab);
	free(pg);
}






void affiche_graphe (gra_lsp* pg) {
	// affiche les voisinages de chaque sommet du graphe *pg
	for (int i = 0; i < pg->n; i++){
		printf("%d :  ",i);
		if ((pg->tab)[i] != NULL)
			affiche_liste ((pg->tab)[i]);
		printf("\n");
	}
	printf("\n");
}





void ajoute_arc_p (gra_lsp* pg, int a, int b, float p) {
	// hyp : 0<=a && a<pg->n && 0<=b && b<pg->n
	// modifie les listes d'adjacence du graphe *pg pour y ajouter l'arc {a,b} de poids p
	if ( ((pg->tab)[a] == NULL) || ! contient_sommet((pg->tab)[a],b) ){
		ajoute_debut( &((pg->tab)[a]),b, p );
	}
	//sinon l'arc est déjà présent, il ne faut pas l'ajouter à nouveau
	//sans quoi on introduit des doublons dans les successeurs de a
	}





struct s_arc_p {
	int s1; //sommet origine
	int s2; //sommet destination
	float p; //poids
};

typedef struct s_arc_p arc_p;




void ajoute_arcs_from_tab (gra_lsp* pg, int nba, arc_p* t) {
	//hyp : t est de taille nba et contient des triplet dont les 2 1eres
	// composantes sont des entiers entre 0 et pg->n -1
	// modifie les listes d'adjacence du graphe *pg pour y ajouter les
	// arcs pondérés correspondant aux éléments de t
	for (int i=0; i < nba; i++){
		ajoute_arc_p (pg, t[i].s1, t[i].s2, t[i].p); 
	}
}






int nb_sommets(gra_lsp* pg){
	// retourne le nombre de sommets du graphe *pg codé par mat. d'adj.
	return pg->n;
}






bool est_arc(gra_lsp* pg, int a, int b) {
	// hyp : 0<=a && a<pg->n && 0<=b && b<pg->n
	// teste si (a,b)  est un arc du graphe *pg codé par mat. d'adj.
	return (pg->tab)[a] != NULL && contient_sommet ((pg->tab)[a],b);
}





int nb_succ(gra_lsp* pg, int a) {
	// hyp : 0<=a && a<pg->n
	// retourne le nb de successeurs de a ds le graphe *pg codé par listes d'adj.
	if((pg->tab)[a] != NULL)
		return nb_elem((pg->tab)[a]);
	else
		return 0;
}


int nb_arcs(gra_lsp* pg) {
	// retourne le nombre d'arêtes du graphe *pg codé par mat. d'adj.
	int s = 0;
	for (int i=0; i < pg->n; i++){ 
		if((pg->tab)[i] != NULL)
			s = s + nb_elem((pg->tab)[i]);
	}
	return s;
}

int main(){
	gra_lsp* pg1 = cree_graphe_vide(10);
	affiche_graphe (pg1);
	ajoute_arc_p (pg1,1,2,3.5);
	affiche_graphe (pg1);
	
	arc_p t[6]={{1,3,0.5},{5,6,0.8},{2,4,12.3},{6,8,5.6},{8,6,12},{6,7,13.0}};
	ajoute_arcs_from_tab(pg1,6,t);
	affiche_graphe (pg1);
	
	printf("Le graphe a %d sommets et %d arcs\n",nb_sommets(pg1),nb_arcs(pg1));
	printf("Le sommet 2 a %d successeurs\n",nb_succ(pg1,2));
	printf("Le sommet 3 a %d successeurs\n",nb_succ(pg1,3));
	printf("Le sommet 0 a %d successeurs\n",nb_succ(pg1,0));
	

	supprime_graphe(pg1);
	return 0;
}
