#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <stdbool.h>

#include "liste_ch_int_float.h"

cellule* create_cell(int sommet, float poids){
	//retourne un pointeur vers une cellule 
	//contenant sommet et poids et pointant vers NULL
	//attention allocation de mémoire dynamique
	//penser à libérer le pointeur retourné après usage	
	cellule* res = (cellule*) malloc(sizeof(cellule));
	(*res).s = sommet;
	(*res).p = poids;
	(*res).next = NULL;
	return res;
}



void affiche_liste(liste_c l){
	//hyp : l n'est pas NULL
	//affiche les éléments de la liste l
	cellule* p = l;
	//afficher l'elem de la 1ere cellule
	printf("%d (%f)  ", p->s, p->p);
	//tant qu'il y a une cellule suivante, 
	while(p->next != NULL){
		//s'y déplacer et l'afficher
		p = p->next;
		printf("%d (%f)  ", p->s, p->p);
	}
};



int nb_elem(liste_c l){
	//hyp : l n'est pas NULL
	//retourne le nombre d'élém de l
	//c'est aussi son nb de cellules
	cellule* p = l;
	int res = 1;
	while(p->next != NULL){
		p = p->next;
		res = res + 1;
	}
	return res;
}



liste_c create_from_tabs(int* tab_sommets,float* tab_poids, int lg){
	// tab_sommets et tab_poids sont de taille lg>0
	// retourne une liste chainée contenant les éléments de tab
	// attention allocation dynamique de mémoire
	// utiliser free_liste_c sur le pointeur retourné après usage
	
	cellule* res = create_cell(tab_sommets[0],tab_poids[0]);
	cellule* prec = res;
	int i = 1;
	while(i < lg){
		prec->next = create_cell(tab_sommets[i],tab_poids[i]);
		prec = prec->next;
		i = i + 1;
	}
	return res;
}



void free_liste_c(liste_c l){
	//hyp : l n'est pas NULL
	//libère la mémoire occupée par les cellules de l
	cellule* p = l;
	cellule* temp;
	while(p->next != NULL){
		temp = p->next;
		free(p);
		p = temp;
	}
}



bool contient_sommet(liste_c l, int sommet){
	//hyp : l n'est pas NULL
	//retourne si x est un élément de la liste l
	cellule* p = l;
	while(true){
		if(sommet == p->s)
			return true;
		else if (p->next == NULL)
			return false;
		else
			p = p->next;
	}
};



void ajoute_debut(liste_c* ptr_l, int sommet, float poids){
	//met à jour la liste *ptr_l passée par référence
	//pour que ce soit la liste l devant laquelle on a ajouté x
	cellule* res = create_cell(sommet, poids);
	res->next = (*ptr_l);
	*ptr_l = res;
}

